/* IMPORT */
import { isWeakReferable } from 'is';
/* HELPERS */
const isWeakabe = (value) => {
    return isWeakReferable(value);
};
/* MAIN */
class MildMap {
    /* VARIABLES */
    #strong;
    #weak;
    #size;
    #finalizationRegistry;
    #finalizationTokens;
    /* CONSTRUCTOR */
    constructor(entries) {
        /* VARIABLES */
        this.#strong = new Map();
        this.#weak = new WeakMap();
        this.#size = 0;
        this.#finalizationRegistry = new FinalizationRegistry(() => this.#size -= 1);
        this.#finalizationTokens = new WeakMap();
        if (entries) {
            for (const [key, value] of entries) {
                this.set(key, value);
            }
        }
    }
    /* GETTER API */
    get size() {
        return this.#size;
    }
    /* API */
    delete(key) {
        const hasKey = this.has(key);
        if (!hasKey)
            return false;
        this.#size -= 1;
        if (isWeakabe(key)) {
            const token = this.#finalizationTokens.get(key);
            if (token) {
                this.#finalizationRegistry.unregister(token);
                this.#finalizationTokens.delete(key);
            }
            return this.#weak.delete(key);
        }
        else {
            return this.#strong.delete(key);
        }
    }
    get(key) {
        if (isWeakabe(key)) {
            return this.#weak.get(key);
        }
        else {
            return this.#strong.get(key);
        }
    }
    has(key) {
        if (isWeakabe(key)) {
            return this.#weak.has(key);
        }
        else {
            return this.#strong.has(key);
        }
    }
    set(key, value) {
        const hasKey = this.has(key);
        if (!hasKey) {
            this.#size += 1;
        }
        if (isWeakabe(key)) {
            this.#weak.set(key, value);
            if (!hasKey) {
                const token = {};
                this.#finalizationRegistry.register(key, token, token);
                this.#finalizationTokens.set(key, token);
            }
        }
        else {
            this.#strong.set(key, value);
        }
        return this;
    }
}
/* EXPORT */
export default MildMap;
